/**
 * Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You
 * may not use this file except in compliance with the License. A copy of
 * the License is located at
 *
 *     http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 * ANY KIND, either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 */

module.exports = {
  format: 'query',
  apiVersion: '2012-11-05',
  endpointPrefix: 'sqs',
  resultWrapped: true,
  serviceAbbreviation: 'Amazon SQS',
  serviceFullName: 'Amazon Simple Queue Service',
  signatureVersion: 'v4',
  timestampFormat: 'iso8601',
  operations: {
    addPermission: {
      name: 'AddPermission',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          Label: {
            required: true
          },
          AWSAccountIds: {
            type: 'list',
            members: {
              name: 'AWSAccountId'
            },
            flattened: true,
            required: true
          },
          Actions: {
            type: 'list',
            members: {
              name: 'ActionName'
            },
            flattened: true,
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
        }
      }
    },
    changeMessageVisibility: {
      name: 'ChangeMessageVisibility',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          ReceiptHandle: {
            required: true
          },
          VisibilityTimeout: {
            type: 'integer',
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
        }
      }
    },
    changeMessageVisibilityBatch: {
      name: 'ChangeMessageVisibilityBatch',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          Entries: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                  required: true
                },
                ReceiptHandle: {
                  required: true
                },
                VisibilityTimeout: {
                  type: 'integer'
                }
              },
              name: 'ChangeMessageVisibilityBatchRequestEntry'
            },
            flattened: true,
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          ChangeMessageVisibilityBatchResultEntry: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                }
              },
              name: 'ChangeMessageVisibilityBatchResultEntry'
            },
            flattened: true,
            name: 'Successful'
          },
          BatchResultErrorEntry: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                },
                SenderFault: {
                  type: 'boolean'
                },
                Code: {
                },
                Message: {
                }
              },
              name: 'BatchResultErrorEntry'
            },
            flattened: true,
            name: 'Failed'
          }
        }
      }
    },
    createQueue: {
      name: 'CreateQueue',
      input: {
        type: 'structure',
        members: {
          QueueName: {
            required: true
          },
          Attributes: {
            type: 'map',
            keys: {
              name: 'Name'
            },
            members: {
              name: 'Value'
            },
            flattened: true,
            name: 'Attribute'
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          QueueUrl: {
          }
        }
      }
    },
    deleteMessage: {
      name: 'DeleteMessage',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          ReceiptHandle: {
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
        }
      }
    },
    deleteMessageBatch: {
      name: 'DeleteMessageBatch',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          Entries: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                  required: true
                },
                ReceiptHandle: {
                  required: true
                }
              },
              name: 'DeleteMessageBatchRequestEntry'
            },
            flattened: true,
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          DeleteMessageBatchResultEntry: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                }
              },
              name: 'DeleteMessageBatchResultEntry'
            },
            flattened: true,
            name: 'Successful'
          },
          BatchResultErrorEntry: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                },
                SenderFault: {
                  type: 'boolean'
                },
                Code: {
                },
                Message: {
                }
              },
              name: 'BatchResultErrorEntry'
            },
            flattened: true,
            name: 'Failed'
          }
        }
      }
    },
    deleteQueue: {
      name: 'DeleteQueue',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
        }
      }
    },
    getQueueAttributes: {
      name: 'GetQueueAttributes',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          AttributeNames: {
            type: 'list',
            members: {
              name: 'AttributeName'
            },
            flattened: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          Attribute: {
            type: 'map',
            keys: {
              name: 'Name'
            },
            members: {
              name: 'Value'
            },
            flattened: true,
            name: 'Attributes'
          }
        }
      }
    },
    getQueueUrl: {
      name: 'GetQueueUrl',
      input: {
        type: 'structure',
        members: {
          QueueName: {
            required: true
          },
          QueueOwnerAWSAccountId: {
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          QueueUrl: {
          }
        }
      }
    },
    listQueues: {
      name: 'ListQueues',
      input: {
        type: 'structure',
        members: {
          QueueNamePrefix: {
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          QueueUrl: {
            type: 'list',
            members: {
              name: 'QueueUrl'
            },
            flattened: true,
            name: 'QueueUrls'
          }
        }
      }
    },
    receiveMessage: {
      name: 'ReceiveMessage',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          AttributeNames: {
            type: 'list',
            members: {
              name: 'AttributeName'
            },
            flattened: true
          },
          MaxNumberOfMessages: {
            type: 'integer'
          },
          VisibilityTimeout: {
            type: 'integer'
          },
          WaitTimeSeconds: {
            type: 'integer'
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          Message: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                MessageId: {
                },
                ReceiptHandle: {
                },
                MD5OfBody: {
                },
                Body: {
                },
                Attribute: {
                  type: 'map',
                  keys: {
                    name: 'Name'
                  },
                  members: {
                    name: 'Value'
                  },
                  flattened: true,
                  name: 'Attributes'
                }
              },
              name: 'Message'
            },
            flattened: true,
            name: 'Messages'
          }
        }
      }
    },
    removePermission: {
      name: 'RemovePermission',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          Label: {
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
        }
      }
    },
    sendMessage: {
      name: 'SendMessage',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          MessageBody: {
            required: true
          },
          DelaySeconds: {
            type: 'integer'
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          MD5OfMessageBody: {
          },
          MessageId: {
          }
        }
      }
    },
    sendMessageBatch: {
      name: 'SendMessageBatch',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          Entries: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                  required: true
                },
                MessageBody: {
                  required: true
                },
                DelaySeconds: {
                  type: 'integer'
                }
              },
              name: 'SendMessageBatchRequestEntry'
            },
            flattened: true,
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
          SendMessageBatchResultEntry: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                },
                MessageId: {
                },
                MD5OfMessageBody: {
                }
              },
              name: 'SendMessageBatchResultEntry'
            },
            flattened: true,
            name: 'Successful'
          },
          BatchResultErrorEntry: {
            type: 'list',
            members: {
              type: 'structure',
              members: {
                Id: {
                },
                SenderFault: {
                  type: 'boolean'
                },
                Code: {
                },
                Message: {
                }
              },
              name: 'BatchResultErrorEntry'
            },
            flattened: true,
            name: 'Failed'
          }
        }
      }
    },
    setQueueAttributes: {
      name: 'SetQueueAttributes',
      input: {
        type: 'structure',
        members: {
          QueueUrl: {
            required: true
          },
          Attributes: {
            type: 'map',
            keys: {
              name: 'Name'
            },
            members: {
              name: 'Value'
            },
            flattened: true,
            name: 'Attribute',
            required: true
          }
        }
      },
      output: {
        type: 'structure',
        members: {
        }
      }
    }
  },
  pagination: {
    listQueues: {
      resultKey: 'QueueUrls'
    }
  }
};
